import grpc
import binascii
import requests
import os

_SIGNATURE_HEADER_KEY = 'x-signature'
_auth_host = 'https://auth.pravah.io'
_auth_port = '5000'

class AuthGateway(grpc.AuthMetadataPlugin):

    def set_auth(self, auth_token, client_id=''):
        #self.client_id = client_id
        self.auth_token = auth_token

    def __call__(self, context, callback):
        """Implements authentication by passing metadata to a callback.
        Implementations of this method must not block.
        Args:
          context: An AuthMetadataContext providing information on the RPC that
            the plugin is being called to authenticate.
          callback: An AuthMetadataPluginCallback to be invoked either
            synchronously or asynchronously.
        """
        # Example AuthMetadataContext object:
        # AuthMetadataContext(
        #     service_url=u'https://localhost:50051/helloworld.Greeter',
        #     method_name=u'SayHello')
        
        real_path = os.path.join(os.path.dirname(__file__), 'chain.pravah.io.crt')
        res = requests.post(_auth_host + ':' + _auth_port + '/token', json={
	        "authentication_token": self.auth_token
        }, timeout=15, verify=real_path)

        resJSON = res.json()
        err = resJSON.get('error', None)
        if err != None:
          print('Err in obtaining access token: ' + err)
          return
        
        cnt = resJSON.get('token', None)
        if cnt == None:
          print('No token found.')
          return

        callback(((_SIGNATURE_HEADER_KEY, cnt),), None)


import grpc

from .lib.rpc_pb2 import PeerTopicInfo, PublishData, Data
from .lib.rpc_pb2_grpc import MeshStub
from .exp import MeshRPCException
from .util import getTopicsFromGeospace
from . import crypto
from .auth_gateway import AuthGateway

class MeshRPC:
    def __init__(self, endpoint, auth_token):
        pem = crypto._load_credential_from_file('ca.rpc.pravah.io.crt')
        
        gateway = AuthGateway()
        gateway.set_auth(auth_token)
        
        channel_credential = grpc.ssl_channel_credentials(pem)
        call_credential = grpc.metadata_call_credentials(gateway, 'Auth Gateway')
        composite_credentials = grpc.composite_channel_credentials(
            channel_credential,
            call_credential
        )
        c = grpc.secure_channel(endpoint, composite_credentials)
        #c = grpc.insecure_channel(endpoint)
        self.stub = MeshStub(c)

    def subscribe(self, channel, geospace):
        topicList = getTopicsFromGeospace(channel, geospace)

        m = PeerTopicInfo()
        m.topics.extend(topicList)
        
        s = self.stub.Subscribe(m)

        return s
    
    def unsubscribe(self, channel, geospace):
        topicList = getTopicsFromGeospace(channel, geospace)

        m = PeerTopicInfo()
        m.topics.extend(topicList)

        return self.stub.Unsubscribe(m)

    def registerToPublish(self, channel, geospace):
        topicList = getTopicsFromGeospace(channel, geospace)

        m = PeerTopicInfo()
        m.topics.extend(topicList)

        try:
            p = self.stub.RegisterToPublish(m)
            return p
        except grpc.RpcError as e:
            raise MeshRPCException(e.details())
    
    def unregisterToPublish(self, channel, geospace):
        topicList = getTopicsFromGeospace(channel, geospace)

        m = PeerTopicInfo()
        m.topics.extend(topicList)

        try:
            p = self.stub.UnregisterToPublish(m)
            return p
        except grpc.RpcError as e:
            raise MeshRPCException(e.details())

    def publish(self, channel, geospace, raw):
        topicList = getTopicsFromGeospace(channel, geospace)

        pd = PublishData()
        
        pd.info.topics.extend(topicList)
        pd.data.raw = raw

        try:
            res = self.stub.Publish(pd)
            return res
        except grpc.RpcError as e:
            raise MeshRPCException(e.details())
